==> prayer-1.0.12.tar.gz: prayer-1.0.12/prayer/cmd_filter.c <==
/* $Cambridge: hermes/src/prayer/prayer/cmd_filter.c,v 1.6 2003/06/03 09:07:52 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void generate_form(struct session *session)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct account *account = session->account;
    struct list_item *item;
    unsigned long count;

    /* Title bar */
    html_start(session, b, "Automatic Mail Filtering");
    html_banner_toolbar(session, b, "filter", 0);

    /* Title line */
    bputs(b, "<h2 align=\"center\">Automatic Mail Filtering</h2>" CRLF);

    if (list_length(account->filter_list) == 0) {
        bputs(b, "No filter actions defined at the current time" CRLF);
    } else {
        bputs(b, "<ol>" CRLF);

        count = 0;
        for (item = account->filter_list->head; item; item = item->next) {
            struct filter *filter = (struct filter *) item;

            switch (filter->type) {
            case FILTER_SENDER:
                bputs(b, "<li><strong>SENDER:</strong>" CRLF);
                bputs(b, "Filter messages from \"");
                html_quote_string(b, filter->local_part);
                bputs(b, " at ");
                html_quote_string(b, filter->domain);
                bputs(b, ".\"" CRLF);
                html_session_bprintf(session, b, "<a href=\"",
                                     "\">Remove this filter.</a>",
                                     "filter/remove/%lu", count);
                break;
            case FILTER_RECIPIENT:
                bputs(b, "<li><strong>RECIPIENT:</strong>" CRLF);
                bputs(b, "Filter messages to \"");
                html_quote_string(b, filter->local_part);
                bputs(b, " at ");
                html_quote_string(b, filter->domain);
                bputs(b, "\"." CRLF);
                html_session_bprintf(session, b, "<a href=\"",
                                     "\">Remove this filter.</a>",
                                     "filter/remove/%lu", count);
                break;
            case FILTER_SUBJECT:
                bputs(b, "<li><strong>SUBJECT:</strong>" CRLF);
                bputs(b, "Filter messages with subject of \"");
                html_quote_string(b, filter->subject);
                bputs(b, "\"." CRLF);
                html_session_bprintf(session, b, "<a href=\"",
                                     "\">Remove this filter.</a>",
                                     "filter/remove/%lu", count);
                break;
            default:
                bputs(b, "<li>Unknown filter type:" CRLF);
            }

            if (filter->mailbox || filter->copy) {
                bputs(b, "<ul>" CRLF);
                if (filter->mailbox) {
                    /* "mail/" hardwired at the moment. */
                    bputs(b, "<li>Store in mail folder: ");
                    if (account->filter_restricted)
                        bputs(b, "mail/");
                    html_quote_string(b, filter->mailbox);
                    bputs(b, "</li>" CRLF);
                }
                if (filter->copy)
                    bputs(b, "<li>Copy message to inbox</li>" CRLF);
                bputs(b, "</ul>" CRLF);
            }
            bputs(b, "</li>" CRLF);
            count++;
        }
        bputs(b, "</ol>" CRLF);
    }

    /* Form body */
    html_form_start(session, b, "POST", "filter");
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);

    /* Filter type */
    bputs(b, "<td>" CRLF);
    bputs(b, "<select name=\"type\" size=1>" CRLF);
    bputs(b, "<option value=\"sender\">Filter on Sender:</option>" CRLF);
    bputs(b,
          "<option value=\"recipient\">Filter on Recipient:</option>"
          CRLF);
    bputs(b, "<option value=\"subject\">Filter on Subject:</option>" CRLF);
    bputs(b, "</select>" CRLF);
    bputs(b, "</td>" CRLF);

    html_form_table_field(b, NIL, "param", NIL, 40);
    html_form_table_submit(b, "sub_add", "Add filter");
    bputs(b, "</tr></table>" CRLF);

    bputs(b, "<table><tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Back to Options Screen");
    bputs(b, "</tr></table>" CRLF);

    html_form_end(b);

    html_end(b);
    response_html(request, 200);        /* Success */
}

static void
generate_error(struct session *session)
{
    struct account *account = session->account;
    struct request *request = session->request;
    char *msg = account_fetch_message(account);
    struct buffer *b = request->write_buffer;

    /* Title Bar */
    html_start(session, b, "Automatic mail filtering");
    html_banner_toolbar(session, b, "filter", 0);

    /* Title line */
    bputs(b,
          "<h2 align=\"center\">Automatic Mail Filtering</h2>"CRLF);

    bputs(b, "<b>Error:</b> ");
    if (msg)
        html_quote_string(b, msg);
    else
        bputs(b, "Unable to check mail processing status");
    bputs(b, "" CRLF);

    /* Form body */
    html_form_start(session, b, "POST", "filter");

    /* Cancel buttons */
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Back to Options Screen");
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);
    html_form_end(b);
    html_end(b);
    response_html(request, 200);
}

void cmd_filter(struct session *session)
{
    struct request *request = session->request;
    struct account *account = session->account;
    unsigned long offset;
    struct assoc *h = NIL;
    struct filter *filter = filter_alloc();
    char *type;
    char *param;

    if (request->method != POST) {
        if (!account_mail_check(account, request->pool)) {
            generate_error(session);
            return;
        }
        if (session->half_filter) {
            filter_free(session->half_filter);
            session->half_filter = NIL;
        }

        if ((request->argc >= 3) && !strcmp(request->argv[1], "remove")) {
            offset = atoi(request->argv[2]);
            account_filter_remove(session->account, offset);
            account_mail_update(session->account, request->pool);
            session_message(session, "Removed filter number: %lu", offset + 1);
        }
        generate_form(session);
        return;
    }

    request_decode_post(request);
    h = request->form;

    if (assoc_lookup(h, "sub_cancel")) {
        session_redirect(session, request, "manage");
        return;
    }

    type = assoc_lookup(h, "type");
    param = assoc_lookup(h, "param");

    if (!(type && type[0])) {
        session_message(session, "Missing type field");
        generate_form(session);
        return;
    }

    if (param && param[0])
        param = string_trim_whitespace(param);
    
    if (!(param && param[0])) {
        session_message(session, "Missing or empty parameter field");
        generate_form(session);
        return;
    }

    if (!strcmp(type, "block") ||
        !strcmp(type, "sender") || !strcmp(type, "recipient")) {
        /* Param is address or address wildcard */
        if (!filter_test_addr(filter, session, param)) {
            session_message(session, "Invalid filter pattern");
            generate_form(session);
            return;
        }
    }

    if (!strcmp(type, "subject")) {
        session_message(session, "Adding SUBJECT filter: \"%s\"", param);
        filter_set_type(filter, FILTER_SUBJECT);
        filter_set_subject(filter, param);
    } else if (!strcmp(type, "sender")) {
        session_message(session, "Adding SENDER filter: \"%s\"", param);
        filter_set_type(filter, FILTER_SENDER);
        filter_set_addr(filter, session, param);
    } else if (!strcmp(type, "recipient")) {
        session_message(session, "Adding RECIPIENT filter: \"%s\"",
                        param);
        filter_set_type(filter, FILTER_RECIPIENT);
        filter_set_addr(filter, session, param);
    } else {
        session_message(session, "Bad/Unknown filter type");
        session_redirect(session, request, "error");
        return;
    }
    
    session->half_filter = filter;
    session_redirect(session, request, "filter_select");
}

==> prayer-1.0.16.tar.gz: prayer-1.0.16/prayer/cmd_filter.c <==
/* $Cambridge: hermes/src/prayer/prayer/cmd_filter.c,v 1.11 2005/08/10 09:48:34 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void generate_form(struct session *session)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct account *account = session->account;
    struct list_item *item;
    unsigned long count;

    /* Title bar */
    html_start(session, b, "Automatic Mail Filtering");
    html_banner_toolbar(session, b, "filter", 0);

    /* Title line */
    bputs(b, "<h2 align=\"center\">Automatic Mail Filtering</h2>" CRLF);

    if (list_length(account->filter_list) == 0) {
        bputs(b, "No filter actions defined at the current time" CRLF);
    } else {
        bputs(b, "<ol>" CRLF);

        count = 0;
        for (item = account->filter_list->head; item; item = item->next) {
            struct filter *filter = (struct filter *) item;

            switch (filter->type) {
            case FILTER_SENDER:
                bputs(b, "<li><strong>SENDER:</strong>" CRLF);
                bputs(b, "Filter messages from \"");
                html_quote_string(b, filter->local_part);
                bputs(b, " at ");
                html_quote_string(b, filter->domain);
                bputs(b, ".\"" CRLF);
                html_session_bprintf(session, b, "<a href=\"",
                                     "\">Remove this filter.</a>",
                                     "filter/remove/%lu", count);
                break;
            case FILTER_RECIPIENT:
                bputs(b, "<li><strong>RECIPIENT:</strong>" CRLF);
                bputs(b, "Filter messages to \"");
                html_quote_string(b, filter->local_part);
                bputs(b, " at ");
                html_quote_string(b, filter->domain);
                bputs(b, "\"." CRLF);
                html_session_bprintf(session, b, "<a href=\"",
                                     "\">Remove this filter.</a>",
                                     "filter/remove/%lu", count);
                break;
            case FILTER_SUBJECT:
                bputs(b, "<li><strong>SUBJECT:</strong>" CRLF);
                bputs(b, "Filter messages with subject of \"");
                html_quote_string(b, filter->subject);
                bputs(b, "\"." CRLF);
                html_session_bprintf(session, b, "<a href=\"",
                                     "\">Remove this filter.</a>",
                                     "filter/remove/%lu", count);
                break;
            default:
                bputs(b, "<li>Unknown filter type:" CRLF);
            }

            if (filter->mailbox || filter->copy) {
                bputs(b, "<ul>" CRLF);
                if (filter->mailbox) {
                    /* "mail/" hardwired at the moment. */
                    bputs(b, "<li>Store in mail folder: ");
                    html_quote_string(b, filter->mailbox);
                    bputs(b, "</li>" CRLF);
                }
                if (filter->copy)
                    bputs(b, "<li>Copy message to inbox</li>" CRLF);
                bputs(b, "</ul>" CRLF);
            }
            bputs(b, "</li>" CRLF);
            count++;
        }
        bputs(b, "</ol>" CRLF);
    }

    /* Form body */
    html_form_start(session, b, "POST", "filter");
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);

    /* Filter type */
    bputs(b, "<td>" CRLF);
    bputs(b, "<select name=\"type\" size=1>" CRLF);
    bputs(b, "<option value=\"sender\">Filter on Sender:</option>" CRLF);
    bputs(b,
          "<option value=\"recipient\">Filter on Recipient:</option>"
          CRLF);
    bputs(b, "<option value=\"subject\">Filter on Subject:</option>" CRLF);
    bputs(b, "</select>" CRLF);
    bputs(b, "</td>" CRLF);

    html_form_table_field(b, NIL, "param", NIL, 40);
    html_form_table_submit(b, "sub_add", "Add filter");
    bputs(b, "</tr></table>" CRLF);

    bputs(b, "<table><tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Back to Options Screen");
    bputs(b, "</tr></table>" CRLF);

    html_form_end(b);

    html_end(b);
    response_html(request, 200);        /* Success */
}

static void
generate_error(struct session *session)
{
    struct account *account = session->account;
    struct request *request = session->request;
    char *msg = account_fetch_message(account);
    struct buffer *b = request->write_buffer;

    /* Title Bar */
    html_start(session, b, "Automatic mail filtering");
    html_banner_toolbar(session, b, "filter", 0);

    /* Title line */
    bputs(b,
          "<h2 align=\"center\">Automatic Mail Filtering</h2>"CRLF);

    bputs(b, "<b>Error:</b> ");
    if (msg) {
        html_quote_string(b, msg);
        session_log(session, "[cmd_filter] %s", msg);
    } else
        bputs(b, "Unable to check mail processing status");
    bputs(b, "" CRLF);

    /* Form body */
    html_form_start(session, b, "POST", "filter");

    /* Cancel buttons */
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Back to Options Screen");
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);
    html_form_end(b);
    html_end(b);
    response_html(request, 200);
}

void cmd_filter(struct session *session)
{
    struct request *request = session->request;
    struct account *account = session->account;
    unsigned long offset;
    struct assoc *h = NIL;
    struct filter *filter = filter_alloc();
    char *type;
    char *param;

    if (request->method != POST) {
        if (!account_mail_check(account, request->pool)) {
            generate_error(session);
            return;
        }
        if (session->half_filter) {
            filter_free(session->half_filter);
            session->half_filter = NIL;
        }

        if ((request->argc >= 3) && !strcmp(request->argv[1], "remove")) {
            offset = atoi(request->argv[2]);
            account_filter_remove(session->account, offset);
            if (account_mail_update(session->account, request->pool)) {
                session_message(session, "Removed filter number: %lu",
                                offset + 1);
            } else {
                char *msg = account_fetch_message(account);

                session_message(session, "Failed to remove filter number: %lu",
                                offset + 1, msg);
                session_log(session,
                            "[cmd_filter] Failed to remove filter number: %lu",
                            offset + 1, msg);
            }
        }
        generate_form(session);
        return;
    }

    request_decode_post(request);
    h = request->form;

    if (assoc_lookup(h, "sub_cancel")) {
        session_redirect(session, request, "manage");
        return;
    }

    type = assoc_lookup(h, "type");
    param = assoc_lookup(h, "param");

    if (!(type && type[0])) {
        session_message(session, "Missing type field");
        generate_form(session);
        return;
    }

    if (param && param[0])
        param = string_trim_whitespace(param);
    
    if (!(param && param[0])) {
        session_message(session, "Missing or empty parameter field");
        generate_form(session);
        return;
    }

    if (!strcmp(type, "block") ||
        !strcmp(type, "sender") || !strcmp(type, "recipient")) {
        /* Param is address or address wildcard */
        if (!filter_test_addr(filter, session, param)) {
            session_message(session, "Invalid filter pattern");
            generate_form(session);
            return;
        }
    }

    if (!strcmp(type, "subject")) {
        session_message(session, "Adding SUBJECT filter: \"%s\"", param);
        filter_set_type(filter, FILTER_SUBJECT);
        filter_set_subject(filter, param);
    } else if (!strcmp(type, "sender")) {
        session_message(session, "Adding SENDER filter: \"%s\"", param);
        filter_set_type(filter, FILTER_SENDER);
        filter_set_addr(filter, session, param);
    } else if (!strcmp(type, "recipient")) {
        session_message(session, "Adding RECIPIENT filter: \"%s\"",
                        param);
        filter_set_type(filter, FILTER_RECIPIENT);
        filter_set_addr(filter, session, param);
    } else {
        session_message(session, "Bad/Unknown filter type");
        session_redirect(session, request, "error");
        return;
    }
    
    session->half_filter = filter;
    session_redirect(session, request, "filter_select");
}
